#include <EEPROM.h>
#include <Adafruit_GFX.h>
#include "src/TFTLib/Adafruit_TFTLCD_8bit_STM32.h"
#include "global.h"
#include "variables.h"
#include <SoftWire.h>

SoftWire SWire(I2C_SCL, I2C_SDA, SOFT_STANDARD);
#define HTU21D_I2C_ADDRESS  0x40
#define HTU21D_REQ_TEMP 0xF3
#define HTU21D_REQ_HUM  0xF5


#define FIRMWARE_VERSION	"1.0"

// ------------------------
void setup()	{
   
  afio_cfg_debug_ports(AFIO_DEBUG_NONE);

  //delay(2000); // Wartezeit damit USB-CDC bereit ist
	DBG_INIT(SERIAL_BAUD_RATE);
	DBG_PRINT("ELV Display, version ");
	DBG_PRINTLN(FIRMWARE_VERSION);

	// set digital and analog stuff
	initIO();
	
	// load scope config or factory reset to defaults
	//loadConfig(digitalRead(BTN4) == LOW);
	
	// init the IL9341 display
	initDisplay();

  SWire.begin();
}


int32_t i2cGetTemperature(uint8_t i2cAddress)
{
   int32_t temp;
   uint8_t i;
   uint16_t value;
   uint8_t dat[3];
  
   SWire.beginTransmission(i2cAddress);
   SWire.write(HTU21D_REQ_TEMP);
   SWire.endTransmission();
   delay(60);
   SWire.requestFrom(i2cAddress, 2);
   i = 0;
   while (SWire.available() && (i < 3))
   {
      dat[i] = SWire.read();
      Serial.print(dat[i], HEX);
      Serial.print(" ");
      i++;
   }
    value = ((uint16_t)dat[0] << 8) | dat[1];
    value &= ~0x03;
    // Temp = -46.85 + 175.72 * S_TEMP / 2^16
    temp = ((17572 * (uint32_t)value) >> 16) - 4685;

   return(temp);
}


int32_t i2cGetHumidity(uint8_t i2cAddress)
{
   int32_t hum;
   uint8_t i;
   uint16_t value;
   uint8_t dat[3];

    SWire.beginTransmission(HTU21D_I2C_ADDRESS);
    SWire.write(HTU21D_REQ_HUM);
    SWire.endTransmission();
    delay(60);
    SWire.requestFrom(HTU21D_I2C_ADDRESS, 3);
    i = 0;
    while(SWire.available() && (i < 3))
    {
      dat[i] = SWire.read();
      Serial.print(dat[i], HEX);
      Serial.print(" ");
      i++;
    }
    value = ((uint16_t)dat[0] << 8) | dat[1];
    value &= ~0x03;
    Serial.print("  ");
    Serial.print(value);
    Serial.print("  ");
    // RH = -6 + 125 * S_RH / 2^16
    hum = ((125 * (uint32_t)value) >> 16) - 6;
    if(hum < 0)
    {
      hum = 0;
    }
    if(hum > 99)
    {
      hum = 99;
    }

   return(hum);
}
   

// ------------------------
void loop()	{
// ------------------------
  int32_t temp;
  uint16_t hum;
  
  ELV_Display();
  DBG_PRINTLN("ELV Display");

  temp = i2cGetTemperature(HTU21D_I2C_ADDRESS);
  hum = i2cGetHumidity(HTU21D_I2C_ADDRESS);
  showTemperature(temp); 
  showHumidity(hum); 

  delay(1000);
}
